local carPh = ac.accessCarPhysics()

local function dampersR(i)

  local jumpBump = 0

  local postfix = ({'LF', 'RF', 'LR', 'RR'})[i + 1]
  local dampSlowSetup = ac.getScriptSetupValue('DAMP_BUMP_%s' % postfix) or error('Add DAMP_BUMP_%s' % postfix)
  local dampFastSetup = ac.getScriptSetupValue('DAMP_FAST_BUMP_%s' % postfix) or error('Add DAMP_FAST_BUMP_%s' % postfix)
  local reboundSlowSetup = ac.getScriptSetupValue('DAMP_REBOUND_%s' % postfix) or error('Add DAMP_REBOUND_%s' % postfix)
  local reboundFastSetup = ac.getScriptSetupValue('DAMP_FAST_REBOUND_%s' % postfix) or error('Add DAMP_FAST_REBOUND_%s' % postfix)

  local function reset()
  end

  local function update()
    local jumpMult = math.saturate(jumpBump) * 4000

    if i < 2 then
        local damper_front = ac.accessCarDamper(i)

        damper_front.bumpSlow = dampSlowSetup.value + jumpMult
        damper_front.bumpFast = dampFastSetup.value + jumpMult
        damper_front.reboundSlow = reboundSlowSetup.value
        damper_front.reboundFast = reboundFastSetup.value

        if carPh.wheels[i].suspensionTravel > 0.16 then
            damper_front.bumpSlow = 13000 + jumpMult
            damper_front.bumpFast = 10000 + jumpMult
        end
    else
        local damper_rear = ac.accessCarDamper(i)

        damper_rear.bumpSlow = dampSlowSetup.value + jumpMult
        damper_rear.bumpFast = dampFastSetup.value + jumpMult
        damper_rear.reboundSlow = reboundSlowSetup.value
        damper_rear.reboundFast = reboundFastSetup.value

        if carPh.wheels[i].suspensionTravel > 0.14 then
            damper_rear.bumpSlow = 12000 + jumpMult
            damper_rear.bumpFast = 9000 + jumpMult
        end
    end

    jumpBump = carPh.wheels[i].suspensionTravel < 0 and jumpBump + 0.015 or jumpBump
    jumpBump = jumpBump - 0.01

    jumpBump = math.clamp(jumpBump, 0, 2)

    local forcePos = carPh.wheels[i].position
    local forceDir = carPh.transform.up
    local forceAmount = (carPh.wheels[i].suspensionTravel + 0.1) * 7000 * (1 - math.clamp(carPh.wheels[i].suspensionTravel * 50, 0, 1))

    if carPh.wheels[i].suspensionTravel < -10 then
      ac.warn(carPh.wheels[i].suspensionTravel)
    end

    ac.addForce(forcePos, false, forceDir * forceAmount, false)
    ac.addHubForce(i, forcePos, forceDir * -forceAmount, false, false)

  end

  return {
    update = update,
    reset = reset
  }

end

local tyres = { dampersR(0), dampersR(1), dampersR(2), dampersR(3) }

return {
  update = function (dt)
    for i = 0, 3 do
      tyres[i + 1].update()
    end
  end,
  reset = function ()
    for i = 0, 3 do
      tyres[i + 1].reset()
    end
  end
}
